#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Module to combine yearly time series of L3 SSH from CMEMS by track with full timeseries.
Uses xarray module (needs installing in python environment along with 'dask' module)

Created on 07/02/24

@author: ellis e.ash@satoc.eu
"""

#import os
#import time
#import fnmatch
import numpy as np
#import netCDF4
import xarray


outdir = '/Users/ellis/Projects/pass-swio/output_data/'
yrdir = '/Users/ellis/Projects/pass-swio/output_data/'
tdir = '/Users/ellis/Projects/pass-swio/altimetry_along_track/'

print('\nDirectory listing (change in code if necessary): \n\nOutput files: %s \nAlong track files: %s \nYearly track files: %s' \
      %(outdir, tdir, yrdir))

satlist = ('j3','s3a','s3b')
satid = ''

while (satid not in satlist): 
    satid = input("Give satellite id [j3 / s3a / s3b]: ")
year = input("Give year to add to full time series [2022]: ")
if year == '': year = 2022
else: year = int(year)

# list of tracks within data files:    
jtracks = (5,18,29,44,55,68,81,94,105,120,131,146,157,170,196,207,222,233,246)
s3tracks = (12,27,40,55,68,83,96,111,126,141,154,169,182,197,210,225,240,255, \
             268,283,296,311,339,354,369,382,397,410,425,455,468,483,496, 511, \
             524,539,569,582,597,610,625,638,653,668,683,696,711,724,739,752,767)
    
prefix = {'j3': 'j1j2j3_cmems_p', 's3a': 's3a_cmems_p', 's3b': 's3b_cmems_p'}
tracknos = {'j3': jtracks, 's3a': s3tracks, 's3b': s3tracks}

count = 0

for track in sorted(tracknos[satid]):
    
    # for each track combine yearly time series with full time series
    
    yrfile = yrdir + prefix[satid] + '%04d_%04d.nc' % (track, year)
    tfile = tdir + prefix[satid] + '%04d.nc' % track
    outfile = outdir + prefix[satid] + '%04d_upto_%04d.nc' % (track, year)
    
    if (count == 0): print('\nCombining these files (first track example): \n%s\n%s\n\nto give:\n%s' \
        %(tfile,yrfile,outfile))
    
    ds = xarray.open_mfdataset((tfile,yrfile),combine = 'by_coords', concat_dim="time")

    # select unique times in case of duplication:   
    _,index = np.unique(ds['time'], return_index = True)
    
    ds.time.encoding['units'] = 'seconds since 2000-01-01'
    ds.time.encoding['_FillValue'] = False
    ds.lat.encoding['_FillValue'] = False
    ds.lon.encoding['_FillValue'] = False
    
    ds.isel(time=index).to_netcdf(outfile)
    
    count += 1


def main():
    
    print('\nMain block not in use')
    

if __name__ == "__main__":
    
    main()
    
    
    