import numpy as np
#import cPickle
import pickle
import warnings
from tabulate import tabulate

def extractStats(loni,lati,mission):
    assert loni<=180, "loni must be in the range [-180,180]"
    path2Data = '/Shared/Software/Windows/SeaLevel/SeaLevelValidation/data/'
    with open(path2Data + 'statsAltiTracks_' + mission, 'rb') as myFile:
        #stats = cPickle.load(myFile)
        stats = pickle.load(myFile)
    passes = stats.keys()
         
    x = []
    y = []
    Aa = []
    Asa = []
    Pa = []
    Psa = []
    trend = []
    sd = []
    s_Aa = []
    s_Asa = []
    s_Pa = []
    s_Psa = []
    s_trend = []
    for p in passes:
        x.extend(stats[p]["lon"].tolist())
        y.extend(stats[p]["lat"].tolist())
        Aa.extend((stats[p]["amp_an"][:,0]*100).tolist())
        s_Aa.extend((stats[p]["amp_an"][:,1]*100).tolist())
        Asa.extend((stats[p]["amp_semian"][:,0]*100).tolist())
        s_Asa.extend((stats[p]["amp_semian"][:,1]*100).tolist())
        Pa.extend(stats[p]["phase_an"][:,0].tolist())
        s_Pa.extend(stats[p]["phase_an"][:,1].tolist())
        Psa.extend(stats[p]["phase_semian"][:,0].tolist())
        s_Psa.extend(stats[p]["phase_semian"][:,1].tolist())     
        trend.extend(stats[p]["trend"][:,0].tolist())
        s_trend.extend(stats[p]["trend"][:,1].tolist())      
        sd.extend((stats[p]["sdev"]*100).tolist())
            
    # find closest point
    x = np.asarray(x)
    y = np.asarray(y)
    d = np.sqrt((x - loni)**2 + (y - lati)**2)
    kx = np.argsort(d)[0]
    assert d.min()<5, "This point is likely outside the altimetry data domain"
    
    print("             ")
    print(tabulate([['Amplitude of the annual cycle', f'{Aa[kx]:.1f}' + " " + u"\u00B1" + " " + f'{s_Aa[kx]:.1f}','cm'],
     ['Phase of the annual cycle', f'{Pa[kx]:.1f}' + " " + u"\u00B1" + " " + f'{s_Pa[kx]:.1f}','day'],
     ['Amplitude of the semi-annual cycle', f'{Asa[kx]:.1f}' + " " + u"\u00B1" + " " + f'{s_Asa[kx]:.1f}','cm'],
     ['Phase of the semi-annual cycle', f'{Psa[kx]:.1f}' + " " + u"\u00B1" + " " + f'{s_Psa[kx]:.1f}','day'],
     ['Linear trend', f'{trend[kx]:.1f}' + " " + u"\u00B1" + " " + f'{s_trend[kx]:.1f}','mm/yr'],
     ['SD of detrended and non-seasonal sea levels', f'{sd[kx]:.1f}','cm']], headers=['Name of statistics',
     'Estimate' + " " + u"\u00B1" + " 1-sigma", 'Units'], tablefmt='orgtbl'))
    print("             ")
    

if __name__ == "__main__":
    import sys
    extractStats(float(sys.argv[1]),float(sys.argv[2]),sys.argv[3])
