import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.cm as cm
#import cPickle
import pickle
import warnings
import matplotlib.cbook
from mpl_toolkits.basemap import Basemap, shiftgrid
mpl.rcParams.update(mpl.rcParamsDefault)

plt.close("all")
plt.ion()

def plotAllTracks(mission):
    # mission : altimetry mission. Choices are: 'j1j2j3' (Jasons), 's3a' (Sentinel-3A), and 's3b' (Sentinel-3B)
    
    lon_min = 10
    lon_max = 63
    lat_min = -40
    lat_max = -8
    
    path2Data = '/Shared/Software/Windows/SeaLevel/SeaLevelValidation/data/'

    with open(path2Data + 'statsAltiTracks_' + mission, 'rb') as myFile:
        #stats = cPickle.load(myFile)
        stats = pickle.load(myFile)
    passes = stats.keys()

    lonM = lon_max
    lonm = lon_min
    latM = lat_max
    latm = lat_min
    lon0 = 0.5*(lonM + lonm)
    lat0 = 0.5*(latM + latm)
    
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=matplotlib.cbook.mplDeprecation)
        m = Basemap(resolution='l',projection='mill',llcrnrlon=lonm,llcrnrlat=latm,\
            urcrnrlon=lonM,urcrnrlat=latM,area_thresh=500.)
            
        fig1 = plt.figure(1,figsize=(20/2.54,16/2.54))
        ax = plt.gca()
        
        # restrict coordinates of passes to plot domain
        x = []
        y = []
        for p in passes:
            xj = stats[p]["lon"]
            yj = stats[p]["lat"]
            ki = np.where((xj>lonm) & (xj<lonM) & (yj>latm) & (yj<latM))[0]
            x.extend(xj[ki].tolist())
            y.extend(yj[ki].tolist())
            
        # compute native x,y coordinates of grid.
        x = np.asarray(x)
        y = np.asarray(y)
        x,y = m(x,y)

        # define parallels and meridians to draw.
        dx = int((lonM - lonm)/4)
        dy = int((latM - latm)/4)
        parallels = np.arange(-90,90,dy)
        meridians = np.arange(0,360,dx)
        m.drawparallels(parallels,labels=[1,0,0,1],linewidth=0.1,fontsize=12)
        m.drawmeridians(meridians,labels=[1,0,0,1],linewidth=0.1,fontsize=12)

        plt.rc('font',size=12)

        # draw coastlines, parallels, meridians.
        m.drawcoastlines(linewidth=0.)
        m.fillcontinents(zorder=0,color='0.7')

        cs = m.scatter(x,y,s=5,c='k',marker='o',cmap=plt.cm.RdBu_r,edgecolor='None',alpha=1)   

        pos1 = ax.get_position() # get the original position 
        pos2 = [pos1.x0 -0.05, pos1.y0,  pos1.width, pos1.height] 
        ax.set_position(pos2) # set a new position
        #fig1.gca().set_aspect(1)
        fig1.tight_layout()
        plt.subplots_adjust(right=0.87)
        
        print("Select a point from the map using the mouse")
        xs,ys = plt.ginput(1)[0]
        m.plot(xs,ys,'o',markersize=15,markerfacecolor='None',markeredgecolor='red', markeredgewidth=2)
        xs,ys = m(xs,ys,inverse=True)
        return xs,ys

if __name__ == "__main__":
    import sys
    plotStatsMap(sys.argv[1])
