import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.cm as cm
#import cPickle
import pickle
import warnings
import matplotlib.cbook
from mpl_toolkits.basemap import Basemap, shiftgrid
mpl.rcParams.update(mpl.rcParamsDefault)

plt.close("all")

def plotStatsMap(lon_min,lon_max,lat_min,lat_max,vari,mission):
    # vari is a string. Possible values are: "amp_an", "phase_an", "amp_semian", "phase_semian", "trend", "sdev"
    path2Data = '/Shared/Software/Windows/SeaLevel/SeaLevelValidation/data/'
    pathFig = '/Shared/Software/Windows/SeaLevel/SeaLevelValidation/figures/'
    with open(path2Data + 'statsAltiTracks_' + mission, 'rb') as myFile:
        #stats = cPickle.load(myFile)
        stats = pickle.load(myFile)
    passes = stats.keys()

    lonM = lon_max
    lonm = lon_min
    latM = lat_max
    latm = lat_min
    lon0 = 0.5*(lonM + lonm)
    lat0 = 0.5*(latM + latm)
    
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=matplotlib.cbook.mplDeprecation)
        m = Basemap(resolution='l',projection='mill',llcrnrlon=lonm,llcrnrlat=latm,\
            urcrnrlon=lonM,urcrnrlat=latM,area_thresh=500.)
            
        sf = 1. + 0.25*(latM - latm)/float(lonM - lonm)
        fig1 = plt.figure(figsize=(16/2.54/sf,16/2.54))
        ax = plt.gca()
        
        # restrict coordinates of passes to plot domain
        x = []
        y = []
        z = []
        se = []
        for p in passes:
            xj = stats[p]["lon"]
            yj = stats[p]["lat"]
            if vari == "sdev":
                zj = stats[p][vari]
                sej = np.zeros(zj.shape)
            else:
                zj = stats[p][vari][:,0]
                sej = stats[p][vari][:,1]
            ki = np.where((xj>lonm) & (xj<lonM) & (yj>latm) & (yj<latM))[0]
            x.extend(xj[ki].tolist())
            y.extend(yj[ki].tolist())
            z.extend(zj[ki].tolist())
            se.extend(sej[ki].tolist())
            
        # compute native x,y coordinates of grid.
        x = np.asarray(x)
        y = np.asarray(y)
        z = np.asarray(z)
        se = np.asarray(se)
        x,y = m(x,y)
        
        if vari == "amp_an" or vari == "amp_semian" or vari == "sdev":
            z = z*100. # convert to meters

        # define parallels and meridians to draw.
        dx = int((lonM - lonm)/4*sf)
        dy = int((latM - latm)/4)
        parallels = np.arange(-90,90,dy)
        meridians = np.arange(0,360,dx)
        m.drawparallels(parallels,labels=[1,0,0,1],linewidth=0.1,fontsize=12)
        m.drawmeridians(meridians,labels=[1,0,0,1],linewidth=0.1,fontsize=12)

        plt.rc('font',size=12)

        # draw coastlines, parallels, meridians.
        m.drawcoastlines(linewidth=0.)
        m.fillcontinents(zorder=0,color='0.7')
        
        # significance of trend
        """
        if vari == "trend":
            ksig = np.abs(z) > 1.96*se
            x = x[ksig]
            y = y[ksig]
            z = z[ksig]
        """
        
        cm,cM = np.percentile(z[z==z],[5,95])
        if vari == "amp_an" or vari == "amp_semian":
            cm = max(0,cm)
            labelc = "Amplitude (cm)"
        elif vari == "phase_an" or vari == "phase_semian":
            if np.mean(z[z==z])<100:
                z[z>300] = z[z>300] - 360
            cm,cM = np.percentile(z[z==z],[5,95])
            labelc = "Phase (days)"
        elif vari == "trend":
            labelc = "Trend (mm/yr)"
        else:
            labelc = "Standard deviation (cm)" 

        cs = m.scatter(x,y,s=5,c=z,marker='o',cmap=plt.cm.RdBu_r,edgecolor='none',alpha=1.,vmin=cm,vmax=cM)   

        cb = m.colorbar(cs,"right", size="5%", pad="2%")
        cb.set_label(labelc)
        """
        if vari == "trend":
            cticks = np.arange(-8,9,2)
            cb.set_ticks(cticks.astype(int))
        #cb.set_clim(cm,cM)
        """

        pos1 = ax.get_position() # get the original position 
        pos2 = [pos1.x0 -0.05, pos1.y0,  pos1.width, pos1.height] 
        ax.set_position(pos2) # set a new position
        fig1.tight_layout()
        plt.subplots_adjust(right=0.87)
        fig1.savefig(pathFig + "stats_" + vari + '_' + mission + ".png",format = 'png', dpi = 200,bbox_inches="tight")
        plt.close("all")

if __name__ == "__main__":
    import sys
    plotStatsMap(float(sys.argv[1]),float(sys.argv[2]),float(sys.argv[3]),float(sys.argv[4]),sys.argv[5],sys.argv[6])
