import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.cm as cm
#import cPickle
import pickle
from mpl_toolkits.basemap import Basemap, shiftgrid
mpl.rcParams.update(mpl.rcParamsDefault)

plt.close("all")

def plot_passes(tgName,xTg,yTg,mission):
    path2Data = '/Shared/Software/Windows/SeaLevel/SeaLevelValidation/data/'
    pathFig = '/Shared/Software/Windows/SeaLevel/SeaLevelValidation/figures/'
    with open(path2Data + 'passes_' + tgName + '_' + mission, 'rb') as myFile:
        #xy = cPickle.load(myFile)
        xy = pickle.load(myFile)
    passes = xy.keys()

    lonM = xTg + 7
    lonm = xTg - 7
    latM = yTg + 7
    latm = yTg - 7
    lon0 = 0.5*(lonM + lonm)
    lat0 = 0.5*(latM + latm)

    m = Basemap(resolution='l',projection='mill',llcrnrlon=lonm,llcrnrlat=latm,\
        urcrnrlon=lonM,urcrnrlat=latM,area_thresh=500.)

    fig1 = plt.figure(figsize=(14/2.54,14/2.54))
    ax = plt.gca()
    
    # restrict coordinates of passes to plot domain
    for p in passes:
        xj = xy[p][:,0]
        yj = xy[p][:,1]
        ki = np.argwhere((xj>lonm+1) & (xj<lonM-1) & (yj>latm+1) & (yj<latM-1))
        q = np.zeros((ki.shape[0],2))
        q[:,0] = xj[ki][:,0]
        q[:,1] = yj[ki][:,0]
        xy[p] = q
    

    # compute native x,y coordinates of grid.
    xp = []
    yp = []
    for p in passes:
        xj = xy[p][:,0]
        yj = xy[p][:,1]
        p2 = [pi for pi in passes if pi != p]
        d = np.zeros((len(p2)+1,))
        dmin = np.zeros((xj.shape[0],))
        for ii in range(xj.shape[0]):
            xjj = xj[ii]
            yjj = yj[ii]
            for i,pi in enumerate(p2):
                xi = xy[pi][:,0]
                yi = xy[pi][:,1]
                di = np.sqrt((xi - xjj)*(xi - xjj) + (yi - yjj)*(yi - yjj))
                d[i] = np.min(di)
            d[-1] = np.sqrt((xjj - xTg)*(xjj - xTg) + (yjj - yTg)*(yjj - yTg))
            dmin[ii] = np.min(d)
            if np.all(d >= 3):
                xj,yj = m(xjj+0.4,yjj)
                break
            if ii == (xj.shape[0]-1):
                kmax = np.argmax(dmin)
                xj,yj = m(xj[kmax]+0.4,yj[kmax])
        xp.append(xj)
        yp.append(yj)
    
    for p in passes:
        xi,yi = m(xy[p][:,0],xy[p][:,1])
        xy[p][:,0] = xi
        xy[p][:,1] = yi

    xp = np.asarray(xp)
    yp = np.asarray(yp)  

    if xTg >= 26:
        xn,yn = m(xTg-3,yTg+0.3)
    else:
        xn,yn = m(xTg+0.5,yTg+0.3)
    xTg,yTg = m(xTg,yTg)

    # define parallels and meridians to draw.
    parallels = np.arange(-80.,0,4.)
    meridians = np.arange(0,360,4.)
    m.drawparallels(parallels,labels=[1,0,0,1],linewidth=0.1,fontsize=14)
    m.drawmeridians(meridians,labels=[1,0,0,1],linewidth=0.1,fontsize=14)

    plt.rc('font',size=14)

    # draw coastlines, parallels, meridians.
    m.drawcoastlines(linewidth=0.)
    m.fillcontinents(zorder=0,color='0.7')

    # draw arrow at Cape Hatteras
    """
    x3, y3 = m(-75.54+360,35.25)
    x4, y4 = m(-73.5+360,35.07)
    plt.annotate('Cape Hatteras', xy=(x3, y3),  xycoords='data',
                    xytext=(x4, y4), textcoords='data',fontsize=12,
                    arrowprops=dict(arrowstyle="->")
                    )
    """
    m.plot(xTg,yTg,color='r',marker='o',linestyle='None',markersize=10,zorder=11)
    #plt.text(xn,yn,tgName.replace("_"," "),color='r',zorder = 10)
    plt.title(tgName.replace("_"," "),fontsize=18)

    # plot passes
    for i,p in enumerate(passes):
        m.plot(xy[p][:,0],xy[p][:,1],color='k',marker='o',linestyle='None',markersize=3)
        plt.text(xp[i],yp[i],p,color='k',zorder = 10)

    pos1 = ax.get_position() # get the original position 
    pos2 = [pos1.x0 -0.05, pos1.y0,  pos1.width, pos1.height] 
    ax.set_position(pos2) # set a new position
    fig1.tight_layout()
    plt.subplots_adjust(right=0.87)
    fig1.savefig(pathFig + tgName + "_passes_" + mission + ".png",format = 'png', dpi = 200,bbox_inches="tight")
    plt.close("all")
