import numpy as np
from os import listdir
#from readAltiData import readAlti
from readAltiData_nc import readAlti #added 27 Feb 2018 (reads Alti from new colocated netcdf data)
import datetime
from seasonalCycle import seasonalCycle
import warnings


def statsAltiTracks(path2Alti,mission,nit = 200):

    listAlti = listdir(path2Alti) # list files in directory
    passes = [x[0:-3].split('p')[-1] for x in listAlti if mission in x and '.nc' in x]
    #passes = [x[0:-3].split('p')[-1] for x in listAlti if ('j1j2j3' in x) or ('n1sa' in x)]
    #mission = [x.split('_')[0] for x in listAlti if ('j1j2j3' in x) or ('n1sa' in x)][0]
    results = {}
    for p in passes:
        
        print("computing statistics for pass: " + str(p))
        # read data
        dati = readAlti(path2Alti,p,mission)
        xa = dati["lon"]
        ya = dati["lat"]
        h = dati["ssha"] + dati["dac"]
        t = dati["time"]
        
        # remove outliers
        with np.errstate(invalid='ignore'):
            with warnings.catch_warnings():
                warnings.simplefilter("ignore", category=RuntimeWarning)
                kout = np.nonzero((np.absolute(h - np.nanmean(h,0)) > 2.)) # remove points ssha > 2 m
                h[kout] = np.NaN
                kout = np.nonzero((np.absolute(h - np.nanmean(h,0)) > 3.*np.nanstd(h,0))) # remove ssha > 3std
                h[kout] = np.NaN
            
        # select records with at least nmin valid observations
        if 'j1' in mission:
            nmin = 500
        else:
            nmin = 24
        knan = ~np.isnan(dati["ssha"])
        knan = np.sum(knan,0)
        knan = knan > nmin
        xa = xa[knan]
        ya = ya[knan]
        h = h[:,knan]
        t = t[:,knan]
        
        # compute statistics
        nt,nx = h.shape
        aai = np.zeros((nx,2))
        asai = np.zeros((nx,2))
        pai = np.zeros((nx,2))
        psai = np.zeros((nx,2))
        tri = np.zeros((nx,2))
        sdev = np.zeros((nx,))
        for i in range(nx):
            if np.mod(i,500) == 0:
                print(str(i) + " of " + str(nx) + " records")
                
            hi = h[:,i]
            ti = t[:,i]
            knan = ~np.isnan(hi) & ~np.isnan(ti)
            stats = seasonalCycle(hi[knan],ti[knan],nit)
            aai[i,0] = stats["amp_an"]
            aai[i,1] = stats["amp_an_se"]
            asai[i,0] = stats["amp_semian"]
            asai[i,1] = stats["amp_semian_se"]
            pai[i,0] = stats["phase_an"]
            pai[i,1] = stats["phase_an_se"]
            psai[i,0] = stats["phase_semian"]
            psai[i,1] = stats["phase_semian_se"]
            tri[i,0] = stats["trend"]
            tri[i,1] = stats["trend_se"]
            res = stats["residual"]
            sdev[i] = np.std(res[res==res])
            
        # store results
        results[p] = {"lon":xa,"lat":ya,"amp_an":aai,"amp_semian":asai,"phase_an":pai,"phase_semian":psai,"trend":tri,
                      "sdev":sdev}
        
    return results
            


            
            




     
